## ====Drop columns from standard definitive screening designs==================
# This script evaluates all projected definitive screening designs
# constructed by dropping columns from standard definitive screening designs.
#
# "Projections of Definitive Screening Designs by Dropping Columns: Selection
# and Evaluation" by Vazquez, A. R., Goos, P., and Schoen, E. D. 
## =============================================================================

# Load source files.
source("code/evaluation_functions.r")

# Specify the standard definitive screening design and the maximum number of
# columns to drop--------------------------------------------------------------
# Number of factors in sDSD.
n <- 12 
# Maximum number of columns to drop.
kmax <- 4


# Construct sDSD using conference matrices-------------------------------------
fileCmat <- paste("conference_matrices/C", n, ".csv", sep = '')
CMat <- read.csv(fileCmat, header = FALSE)
col.names <- paste('C', seq(1, n, 1), sep = ''); colnames(CMat) <- col.names
zrow <- rep(0, times = n)
sDSD <- rbind(rbind(CMat, -1*CMat), zrow)

# Projections of sDSD by dropping columns--------------------------------------
library(snow)
cl <- makeSOCKcluster(c("localhost","localhost")) # For parallel computations.

for (k in 1:kmax){
  
  print(paste('Number of projections of n=', n, ' in k=', k, ': ', choose(n, k),
              sep = '')) 
  projset <- combn(n, k) 
  
  # Compute correlations between pairs of two-factor interaction contrast vectors.
  projDSD <- parApply(cl, projset, MARGIN = 2, ProjStatTwo, sDSD)
  
  # Save results.
  projps <- t(projDSD)
  datsave <- data.frame('Columns' = projps[, 1], 'Average absolute correlation' = projps[, 2], 
                        'Maximum absolute correlation' = projps[, 3], 
                        'Sum of squared correlations' = projps[, 4])
  stsav <- paste('results/CMat', n, 'Drop', k, '.csv', sep = '')
  write.csv(file = stsav, datsave)
  
}

# Identify the best and worst sets of columns using filters in Excel.