heredity <- function(Ds) {
  # Construct the model matrix and link matrix for heredity.
  #
  # INPUTS:
  # Ds  Design (data frame or matrix).
  #
  # OUTPUTS:
  # output: list with the following objects: 
  #                   - M model matrix including the intercept all linear
  #                       main effects, all quadratic main effects, and all
  #                       two factor interaction matrix.
  #                   - R link matrix containing the relation between the
  #                       two-factor interactions and the factors.
  #
  # Note: This function was adapted from the "heredity.R" file in
  #       the supplementary programs of Mee, R.W., Schoen, E.D. and
  #       Edwards, D. (2017). "Selecting an Orthogonal or Nonorthogonal
  #       Two-Level Design for Screening." Technometrics, 59:305-318.
  # --------------------------------------------------------------------
  
  Ds <- as.matrix(Ds)
  n <- dim(Ds)[1]
  m <- dim(Ds)[2]
  fullsize <- m+1
  fullsize <- fullsize+choose(fullsize-1,2) + m
  M <- matrix(0,n,fullsize)
  R <- matrix(0,fullsize,fullsize)
  M[,1] <- matrix(1,n,1)
  M[,2:(m+1)] <- Ds
  M[,(m+2):(2*m+1)] <- Ds^2
  
  R[1,1:(m+1)] <- matrix(1,1,m+1)
  lec <- 2*m+2
  ncols <- choose(lec-2-m,2)
  cols.int <- lec:(lec+ncols-1)
  
  Combos <- combn(2:(lec-1 - m),2)
  a <- M[,Combos[1,]]
  b <- M[,Combos[2,]]
  block <- a*b
  M[,cols.int] <- block
  
  for (j in 1:ncols) {
    cc <- lec+j-1
    R[Combos[,j],cc] <- 1
  }
  R[2:(m+1), (m+2):(2*m+1)] <- 2*diag(m) 
  output <- list()
  output[[1]] <- M
  output[[2]] <- R
  return(output)
}


